@@ -20,7 +20,7 @@
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *)
 
-(* $Id: pcre.ml,v 1.17 2003/12/21 14:46:57 mottl Exp $ *)
+(* $Id: pcre.ml,v 1.19 2004/04/29 13:23:38 mottl Exp $ *)
 
 
 (* Public exceptions and their registration with the C runtime *)
@@ -142,17 +142,31 @@ let rflag_list irflags =
     else flag_list in
   List.fold_left coll [] all_rflags
 
+
 (* Information on the PCRE-configuration (build-time options) *)
 
-external config_utf8 : unit -> bool = "pcre_config_utf8_stub" "noalloc"
-external config_newline : unit -> char = "pcre_config_newline_stub" "noalloc"
-external config_link_size : unit -> int = "pcre_config_link_size_stub" "noalloc"
+external pcre_version : unit -> string = "pcre_version_stub"
 
-external config_match_limit :
+external pcre_config_utf8 : unit -> bool = "pcre_config_utf8_stub" "noalloc"
+
+external pcre_config_newline :
+  unit -> char = "pcre_config_newline_stub" "noalloc"
+
+external pcre_config_link_size :
+  unit -> int = "pcre_config_link_size_stub" "noalloc"
+
+external pcre_config_match_limit :
   unit -> int = "pcre_config_match_limit_stub" "noalloc"
 
-external config_stackrecurse :
-  unit -> int = "pcre_config_stackrecurse_stub" "noalloc"
+external pcre_config_stackrecurse :
+  unit -> bool = "pcre_config_stackrecurse_stub" "noalloc"
+
+let version = pcre_version ()
+let config_utf8 = pcre_config_utf8 ()
+let config_newline = pcre_config_newline ()
+let config_link_size = pcre_config_link_size ()
+let config_match_limit = pcre_config_match_limit ()
+let config_stackrecurse = pcre_config_stackrecurse ()
 
 
 (* Information on patterns *)
@@ -250,7 +264,7 @@ let get_offset_start ovector str_num =
   offset, Array.unsafe_get ovector offset
 
 let get_substring_aux (subj, ovector) offset start =
-  if start < 0 then ""
+  if start < 0 then raise Not_found
   else
     string_unsafe_sub subj start (Array.unsafe_get ovector (offset + 1) - start)
 
@@ -258,15 +272,16 @@ let get_substring (_, ovector as substrings) str_num =
   let offset, start = get_offset_start ovector str_num in
   get_substring_aux substrings offset start
 
-let unsafe_get_substring (_, ovector as substrings) str_num =
-  let offset = str_num lsl 1 in
-  get_substring_aux substrings offset (Array.unsafe_get ovector offset)
-
 let get_substring_ofs (subj, ovector) str_num =
   let offset, start = get_offset_start ovector str_num in
   if start < 0 then raise Not_found
   else start, Array.unsafe_get ovector (offset + 1)
 
+let unsafe_get_substring (_, ovector as substrings) str_num =
+  let offset = str_num lsl 1 in
+  try get_substring_aux substrings offset (Array.unsafe_get ovector offset)
+  with Not_found -> ""
+
 let get_substrings ?(full_match = true) (_, ovector as substrings) =
   if full_match then
     Array.init (Array.length ovector / 3) (unsafe_get_substring substrings)
@@ -274,6 +289,21 @@ let get_substrings ?(full_match = true) (_, ovector as substrings) =
     let len = (Array.length ovector / 3) - 1 in
     Array.init len (fun n -> unsafe_get_substring substrings (n + 1))
 
+let unsafe_get_opt_substring (_, ovector as substrings) str_num =
+  let offset = str_num lsl 1 in
+  try
+    let start = Array.unsafe_get ovector offset in
+    let str = get_substring_aux substrings offset start in
+    Some str
+  with Not_found -> None
+
+let get_opt_substrings ?(full_match = true) (_, ovector as substrings) =
+  if full_match then
+    Array.init (Array.length ovector / 3) (unsafe_get_opt_substring substrings)
+  else
+    let len = (Array.length ovector / 3) - 1 in
+    Array.init len (fun n -> unsafe_get_opt_substring substrings (n + 1))
+
 external get_stringnumber :
   regexp -> string -> int = "pcre_get_stringnumber_stub"
 
@@ -340,10 +370,18 @@ let exec_all ?(iflags = 0) ?flags ?(rex = def_rex) ?pat ?pos ?callout subj =
 let extract ?iflags ?flags ?rex ?pat ?pos ?full_match ?callout subj =
   get_substrings ?full_match (exec ?iflags ?flags ?rex ?pat ?pos ?callout subj)
 
+let extract_opt ?iflags ?flags ?rex ?pat ?pos ?full_match ?callout subj =
+  get_opt_substrings
+    ?full_match (exec ?iflags ?flags ?rex ?pat ?pos ?callout subj)
+
 let extract_all ?iflags ?flags ?rex ?pat ?pos ?full_match ?callout subj =
   let many_sstrs = exec_all ?iflags ?flags ?rex ?pat ?pos ?callout subj in
   Array.map (get_substrings ?full_match) many_sstrs
 
+let extract_all_opt ?iflags ?flags ?rex ?pat ?pos ?full_match ?callout subj =
+  let many_sstrs = exec_all ?iflags ?flags ?rex ?pat ?pos ?callout subj in
+  Array.map (get_opt_substrings ?full_match) many_sstrs
+
 let pmatch ?iflags ?flags ?rex ?pat ?pos ?callout subj =
   try ignore (pcre_exec ?iflags ?flags ?rex ?pat ?pos ?callout subj); true
   with Not_found -> false
@@ -891,13 +929,6 @@ let full_split ?(iflags = 0) ?flags ?(rex = def_rex) ?pat
     List.rev (loop [] (max - 1) pos true)
 
 
-(* Version information *)
-
-external pcre_version : unit -> string = "pcre_version_stub"
-
-let version = pcre_version ()
-
-
 (* Additional convenience functions useful in combination with this library *)
 
 let foreach_line ?(ic = stdin) f =
