@@ -20,7 +20,7 @@
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *)
 
-(* $Id: pcre.mli,v 1.17 2003/12/21 14:46:57 mottl Exp $ *)
+(* $Id: pcre.mli,v 1.19 2004/04/29 13:23:38 mottl Exp $ *)
 
 (** Perl Compatibility Regular Expressions *)
 
@@ -104,22 +104,23 @@ val rflag_list : irflag -> rflag list
 
 (** {6 Information on the PCRE-configuration (build-time options)} *)
 
+(** Version information *)
+val version : string  (** Version of the PCRE-C-library *)
+
 (** Indicates whether UTF8-support is enabled *)
-external config_utf8 : unit -> bool = "pcre_config_utf8_stub" "noalloc"
+val config_utf8 : bool
 
 (** Character used as newline *)
-external config_newline : unit -> char = "pcre_config_newline_stub" "noalloc"
+val config_newline : char
 
 (** Number of bytes used for internal linkage of regular expressions *)
-external config_link_size : unit -> int = "pcre_config_link_size_stub" "noalloc"
+val config_link_size : int
 
 (** Default limit for calls to internal matching function *)
-external config_match_limit :
-  unit -> int = "pcre_config_match_limit_stub" "noalloc"
+val config_match_limit : int
 
 (** Indicates use of stack recursion in matching function *)
-external config_stackrecurse :
-  unit -> int = "pcre_config_stackrecurse_stub" "noalloc"
+val config_stackrecurse : bool
 
 
 (** {6 Information on patterns} *)
@@ -241,11 +242,12 @@ val num_of_subs : substrings -> int
 
 val get_substring : substrings -> int -> string
 (** [get_substring substrings n] @return the [n]th substring
-    (0 is whole match) of [substrings] or the empty string if the
-    corresponding subpattern did not capture a substring.
+    (0 is whole match) of [substrings].
 
     @raise Invalid_argument if [n] is not in the range of the number of
-    substrings. *)
+    substrings.
+    @raise Not_found if the corresponding subpattern did not capture
+           a substring. *)
 
 val get_substring_ofs : substrings -> int -> int * int
 (** [get_substring_ofs substrings n] @return the offset tuple of the
@@ -267,11 +269,24 @@ val get_substrings :
 
     @param full_match default = true *)
 
+val get_opt_substrings :
+  ?full_match : bool ->
+  substrings -> string option array
+(** [get_opt_substrings ?full_match substrings] @return the array of
+    optional substrings in [substrings]. It includes [Some full_match_str]
+    at index 0 when [full_match] is [true], [Some captured_substrings]
+    only when it is [false]. If a subpattern did not capture a substring,
+    [None] is returned in the corresponding position instead.
+
+    @param full_match default = true *)
+
 val get_named_substring : regexp -> string -> substrings -> string
 (** [get_named_substring rex name substrings] @return the named substring
     [name] in regular expression [rex] and [substrings].
 
-    @raise Invalid_argument if there is no such named substring. *)
+    @raise Invalid_argument if there is no such named substring.
+    @raise Not_found if the corresponding subpattern did not capture
+           a substring. *)
 
 val get_named_substring_ofs : regexp -> string -> substrings -> int * int
 (** [get_named_substring_ofs rex name substrings] @return the offset
@@ -424,9 +439,39 @@ val extract :
     @return the array of substrings that match [subj] starting at
     position [pos], using pattern [pat] when given, regular expression
     [rex] otherwise. Uses [flags] when given, the precompiled [iflags]
-    otherwise. It includes the full match at index 0 when [full_match]
-    is [true], the captured substrings only when it is [false]. Callouts
-    are handled by [callout].
+    otherwise. It includes the full match at index 0 when [full_match] is
+    [true], the captured substrings only when it is [false]. Callouts are
+    handled by [callout].  If a subpattern did not capture a substring,
+    the empty string is returned in the corresponding position instead.
+
+    @param iflags default = no extra flags
+    @param flags default = ignored
+    @param rex default = matches whitespace
+    @param pat default = ignored
+    @param pos default = 0
+    @param full_match default = true
+    @param callout default = ignore callouts
+
+    @raise Not_found if pattern does not match. *)
+
+val extract_opt :
+  ?iflags : irflag ->
+  ?flags : rflag list ->
+  ?rex : regexp ->
+  ?pat : string ->
+  ?pos : int ->
+  ?full_match : bool ->
+  ?callout : callout ->
+  string -> string option array
+(** [extract_opt ?iflags ?flags ?rex ?pat ?pos ?full_match ?callout subj]
+    @return the array of optional substrings that match [subj] starting
+    at position [pos], using pattern [pat] when given, regular expression
+    [rex] otherwise. Uses [flags] when given, the precompiled [iflags]
+    otherwise. It includes [Some full_match_str] at index 0 when
+    [full_match] is [true], [Some captured-substrings] only when it is
+    [false]. Callouts are handled by [callout].  If a subpattern did
+    not capture a substring, [None] is returned in the corresponding
+    position instead.
 
     @param iflags default = no extra flags
     @param flags default = ignored
@@ -466,6 +511,36 @@ val extract_all :
 
     @raise Not_found if pattern does not match. *)
 
+val extract_all_opt :
+  ?iflags : irflag ->
+  ?flags : rflag list ->
+  ?rex : regexp ->
+  ?pat : string ->
+  ?pos : int ->
+  ?full_match : bool ->
+  ?callout : callout ->
+  string -> string option array array
+(** [extract_all_opt
+      ?iflags ?flags ?rex ?pat ?pos ?full_match ?callout subj]
+    @return an array of arrays of all optional matching substrings that
+    match [subj] starting at position [pos], using pattern [pat] when
+    given, regular expression [rex] otherwise. Uses [flags] when given,
+    the precompiled [iflags] otherwise. It includes [Some full_match_str]
+    at index 0 of the extracted string arrays when [full_match] is [true],
+    [Some captured_substrings] only when it is [false]. Callouts are
+    handled by [callout].  If a subpattern did not capture a substring,
+    [None] is returned in the corresponding position instead.
+
+    @param iflags default = no extra flags
+    @param flags default = ignored
+    @param rex default = matches whitespace
+    @param pat default = ignored
+    @param pos default = 0
+    @param full_match default = true
+    @param callout default = ignore callouts
+
+    @raise Not_found if pattern does not match. *)
+
 val pmatch :
   ?iflags : irflag ->
   ?flags : rflag list ->
@@ -783,11 +858,6 @@ val full_split :
     @param callout default = ignore callouts *)
 
 
-(** {6 Version information} *)
-
-val version : string  (** Version of the PCRE-C-library *)
-
-
 (** {6 Additional convenience functions} *)
 
 val foreach_line :
